@@ -58,15 +58,6 @@ sub parse {
       : $headers->sec_websocket_location ? 16
       :                                    undef;
 
-    # Don't waste memory
-    if ($self->asset->isa('Mojo::Asset::Memory')) {
-
-        # Upgrade to file storage
-        $self->asset(Mojo::Asset::File->new)
-          if !defined $length
-              || $length > ($ENV{MOJO_MAX_MEMORY_SIZE} || 262144);
-    }
-
     # Content needs to be upgraded to multipart
     if ($self->is_multipart) {
 
@@ -77,9 +68,20 @@ sub parse {
         return Mojo::Content::MultiPart->new($self)->parse;
     }
 
+    # Don't waste memory
+    my $asset = $self->asset;
+    if ($asset->isa('Mojo::Asset::Memory')) {
+
+        # Upgrade to file based storage on demand
+        if ($asset->size > ($ENV{MOJO_MAX_MEMORY_SIZE} || 262144)) {
+            $self->asset(Mojo::Asset::File->new->add_chunk($asset->slurp));
+        }
+    }
+
     # Chunked body or relaxed content
     if ($self->is_chunked || $self->relaxed) {
-        $self->asset->add_chunk($self->buffer->empty);
+        $self->asset->add_chunk($self->{_b2});
+        $self->{_b2} = '';
     }
 
     # Normal body
@@ -89,7 +91,7 @@ sub parse {
         $length ||= $self->headers->content_length || 0;
         my $asset = $self->asset;
         my $need  = $length - $asset->size;
-        $asset->add_chunk($self->buffer->remove($need)) if $need > 0;
+        $asset->add_chunk(substr $self->{_b2}, 0, $need, '') if $need > 0;
 
         # Done
         $self->{_state} = 'done' if $length <= $self->progress;
