@@ -166,8 +166,11 @@ sub _build_tx {
 sub _drop {
     my ($self, $id) = @_;
 
-    # WebSocket
-    if (my $ws = $self->{_cs}->{$id}->{websocket}) { $ws->server_close }
+    # Connection
+    my $c = $self->{_cs}->{$id};
+
+    # Transaction
+    if (my $tx = $c->{websocket} || $c->{transaction}) { $tx->server_close }
 
     # Drop connection
     delete $self->{_cs}->{$id};
@@ -416,12 +419,12 @@ Mojo::Server::Daemon - Async IO HTTP 1.1 And WebSocket Server
 =head1 DESCRIPTION
 
 L<Mojo::Server::Daemon> is a full featured async io HTTP 1.1 and WebSocket
-server with C<IPv6>, C<TLS>, C<Bonjour>, C<epoll>, C<kqueue> and UNIX domain
-socket sharing support.
+server with C<TLS>, C<Bonjour>, C<epoll>, C<kqueue> and UNIX domain socket
+sharing support.
 
-Optional modules L<IO::KQueue>, L<IO::Epoll>, L<IO::Socket::IP>,
-L<IO::Socket::SSL> and L<Net::Rendezvous::Publish> are supported
-transparently and used if installed.
+Optional modules L<IO::KQueue>, L<IO::Epoll>, L<IO::Socket::SSL> and
+L<Net::Rendezvous::Publish> are supported transparently and used if
+installed.
 
 =head1 ATTRIBUTES
 
@@ -458,8 +461,8 @@ Ports and files to listen on, defaults to C<http://*:3000>.
 
 =head2 C<listen_queue_size>
 
-    my $listen_queue_size = $daemon->listen_queue_zise;
-    $daemon               = $daemon->listen_queue_zise(128);
+    my $listen_queue_size = $daemon->listen_queue_size;
+    $daemon               = $daemon->listen_queue_size(128);
 
 Listen queue size, defaults to C<SOMAXCONN>.
 
